/*
 * Copyright 2003 VIA Technologies, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * VIA AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <sys/ioctl.h>
#include <sys/fcntl.h>

int check_num(char *para)
{
  // check the input number is integer.
  int i, len;
  len = strlen(para);
  for ( i = 0 ; i < len ; i++ )
  { 
    if ( para[i] < '0' || para [i] > '9' )
 	return 1;		
  }
  if ( atoi(para) == 0 )
 	return 1;		
  return 0; 
}

int main (int argc, char* argv[])
{
 int HW_RNG ;
 unsigned long total, volume = 1024; 
 int maxvolume;
 int i, check_flag = 1, count = 0, linelen = 64 ;
 char *para;
 char *usage="USAGE: rngtest [-b number] [-n number] [-s]\n"
	     "Option:\n"
	     "       -b: specify how many bytes to generate;\n"
	     "           default 1024; 0 unacceptable; maximum 2,147,483,647\n"
	     "       -n: specify how many hexadecimals per line;\n"
	     "           default 64; 0 unacceptable\n" 
	     "       -s: silent mode; no display output\n"; 

 // Check the program with parameters or not.   
 if ( argc >= 2 )
  {
        argv++;
	count++;
	while ( *argv )
        {
          para = *argv++;
	  count++ ;
          while ( *para )
	  {
	     if ( para[0] != '-' || strlen(para) != 2)
	     {
		printf("%s",usage);
		exit(0);
	     }
	     else
	     {
               // Check the parameter
               switch ( para[1] )
	       {
                case 'b':
			  if ( count > argc )
			    {
		                printf("%s",usage);
				exit(0);				  
			    } 				  
			  volume = atoi(*argv);
                          i = strlen(*argv) ;
                          
                          // Check the number is over than the limitation.
                          if ( i > 10 || ( i == 10 && strcmp(*argv, "2147483647") > 0 ) )
                            {
                              printf("Error:\n");
                              printf("  The number is greater than 2,147,483,647.\n");
                              printf("  Please re-input again!\n");
                              exit(1);
                            }
  			  if ( check_num(*argv++) )
			    {
 				printf("%s",usage);
				exit(0);
			    }
			  break;
               case 'n':
			  if ( count > argc )
			    {
		                printf("%s",usage);
				exit(0);				  
			    }				  
			  linelen = atoi(*argv);
  			  if ( check_num(*argv++) )
			    {
 				printf("%s",usage);
				exit(0);
			    }
                          break;
               case 's':
                          check_flag = 0;
                          break;
                default:
		     	  printf("%s",usage);
			  exit(0);
                          break;
  	       }
	       count++;
	     }
  	     break;					  
	  }
        }
	
  }
 
 int loop = 1;
 i = 1;
 total = volume;

 // Check the input bytes.
 if ( volume > 8193 )
   {
      maxvolume = 8192; 
      loop = (volume / maxvolume) + 1;
   }
 else 
      maxvolume = volume;

 // Open the HW_RNG
 HW_RNG = open ("/dev/hwrandom", O_RDONLY);
 if ( HW_RNG <= 0 )
 {
   printf("Error: Device not ready!!\n");
   exit(0);
 }

 printf("Start to generate %lu bytes.\n", volume);
 while (loop)
   {
     unsigned char buf[maxvolume];
     int size;
     size_t r, off;
 
     size = sizeof(buf);
     off = 0 ;
      
     // Read the data from the HW/RNG
     while ( size &&  ( r = read(HW_RNG, buf + off , size) ) > 0 )
       { 
         off  += r ;
         size -= r ;	
       }
     if (size)
       {
         printf("Error: Read error!\n");
         exit(1);
       }

     // Display hexadecimal or not
     if (check_flag)    
       {
	  for ( i = 0; i < maxvolume; i++)
            {
              if ( linelen == 1 )
                 linelen = 2 ;

              if ( (i % (linelen / 2) )== 0 )
                 printf("\n") ;

              if ( buf[i] > 0xF )
                 printf("%2X", buf[i]);
              else 
                 printf("0%X", buf[i]);

            }
          printf("\n");
       }
     else
          printf("%10d. %d bytes generated.\n", i, maxvolume);
   
    volume -= maxvolume;
    if ( volume < 8193 )
      {
        maxvolume = volume;
      }

    i++;     
    loop-- ; 
  }

 printf("Finished. %lu bytes generated.\n", total);
 close (HW_RNG); 
 return 0;
}
